#define KEY_NULL		0
#define KEY___LINE__		1
#define KEY___FILE__		2
#define KEY___DATA__		3
#define KEY___END__		4
#define KEY_AUTOLOAD		5
#define KEY_BEGIN		6
#define KEY_CORE		7
#define KEY_DESTROY		8
#define KEY_END			9
#define KEY_EQ			10
#define KEY_GE			11
#define KEY_GT			12
#define KEY_LE			13
#define KEY_LT			14
#define KEY_NE			15
#define KEY_abs			16
#define KEY_accept		17
#define KEY_alarm		18
#define KEY_and			19
#define KEY_atan2		20
#define KEY_bind		21
#define KEY_binmode		22
#define KEY_bless		23
#define KEY_caller		24
#define KEY_chdir		25
#define KEY_chmod		26
#define KEY_chomp		27
#define KEY_chop		28
#define KEY_chown		29
#define KEY_chr			30
#define KEY_chroot		31
#define KEY_close		32
#define KEY_closedir		33
#define KEY_cmp			34
#define KEY_connect		35
#define KEY_continue		36
#define KEY_cos			37
#define KEY_crypt		38
#define KEY_dbmclose		39
#define KEY_dbmopen		40
#define KEY_defined		41
#define KEY_delete		42
#define KEY_die			43
#define KEY_do			44
#define KEY_dump		45
#define KEY_each		46
#define KEY_else		47
#define KEY_elsif		48
#define KEY_endgrent		49
#define KEY_endhostent		50
#define KEY_endnetent		51
#define KEY_endprotoent		52
#define KEY_endpwent		53
#define KEY_endservent		54
#define KEY_eof			55
#define KEY_eq			56
#define KEY_eval		57
#define KEY_exec		58
#define KEY_exists		59
#define KEY_exit		60
#define KEY_exp			61
#define KEY_fcntl		62
#define KEY_fileno		63
#define KEY_flock		64
#define KEY_for			65
#define KEY_foreach		66
#define KEY_fork		67
#define KEY_format		68
#define KEY_formline		69
#define KEY_ge			70
#define KEY_getc		71
#define KEY_getgrent		72
#define KEY_getgrgid		73
#define KEY_getgrnam		74
#define KEY_gethostbyaddr	75
#define KEY_gethostbyname	76
#define KEY_gethostent		77
#define KEY_getlogin		78
#define KEY_getnetbyaddr	79
#define KEY_getnetbyname	80
#define KEY_getnetent		81
#define KEY_getpeername		82
#define KEY_getpgrp		83
#define KEY_getppid		84
#define KEY_getpriority		85
#define KEY_getprotobyname	86
#define KEY_getprotobynumber	87
#define KEY_getprotoent		88
#define KEY_getpwent		89
#define KEY_getpwnam		90
#define KEY_getpwuid		91
#define KEY_getservbyname	92
#define KEY_getservbyport	93
#define KEY_getservent		94
#define KEY_getsockname		95
#define KEY_getsockopt		96
#define KEY_glob		97
#define KEY_gmtime		98
#define KEY_goto		99
#define KEY_grep		100
#define KEY_gt			101
#define KEY_hex			102
#define KEY_if			103
#define KEY_index		104
#define KEY_int			105
#define KEY_ioctl		106
#define KEY_join		107
#define KEY_keys		108
#define KEY_kill		109
#define KEY_last		110
#define KEY_lc			111
#define KEY_lcfirst		112
#define KEY_le			113
#define KEY_length		114
#define KEY_link		115
#define KEY_listen		116
#define KEY_local		117
#define KEY_localtime		118
#define KEY_log			119
#define KEY_lstat		120
#define KEY_lt			121
#define KEY_m			122
#define KEY_map			123
#define KEY_mkdir		124
#define KEY_msgctl		125
#define KEY_msgget		126
#define KEY_msgrcv		127
#define KEY_msgsnd		128
#define KEY_my			129
#define KEY_ne			130
#define KEY_next		131
#define KEY_no			132
#define KEY_not			133
#define KEY_oct			134
#define KEY_open		135
#define KEY_opendir		136
#define KEY_or			137
#define KEY_ord			138
#define KEY_pack		139
#define KEY_package		140
#define KEY_pipe		141
#define KEY_pop			142
#define KEY_pos			143
#define KEY_print		144
#define KEY_printf		145
#define KEY_prototype		146
#define KEY_push		147
#define KEY_q			148
#define KEY_qq			149
#define KEY_quotemeta		150
#define KEY_qw			151
#define KEY_qx			152
#define KEY_rand		153
#define KEY_read		154
#define KEY_readdir		155
#define KEY_readline		156
#define KEY_readlink		157
#define KEY_readpipe		158
#define KEY_recv		159
#define KEY_redo		160
#define KEY_ref			161
#define KEY_rename		162
#define KEY_require		163
#define KEY_reset		164
#define KEY_return		165
#define KEY_reverse		166
#define KEY_rewinddir		167
#define KEY_rindex		168
#define KEY_rmdir		169
#define KEY_s			170
#define KEY_scalar		171
#define KEY_seek		172
#define KEY_seekdir		173
#define KEY_select		174
#define KEY_semctl		175
#define KEY_semget		176
#define KEY_semop		177
#define KEY_send		178
#define KEY_setgrent		179
#define KEY_sethostent		180
#define KEY_setnetent		181
#define KEY_setpgrp		182
#define KEY_setpriority		183
#define KEY_setprotoent		184
#define KEY_setpwent		185
#define KEY_setservent		186
#define KEY_setsockopt		187
#define KEY_shift		188
#define KEY_shmctl		189
#define KEY_shmget		190
#define KEY_shmread		191
#define KEY_shmwrite		192
#define KEY_shutdown		193
#define KEY_sin			194
#define KEY_sleep		195
#define KEY_socket		196
#define KEY_socketpair		197
#define KEY_sort		198
#define KEY_splice		199
#define KEY_split		200
#define KEY_sprintf		201
#define KEY_sqrt		202
#define KEY_srand		203
#define KEY_stat		204
#define KEY_study		205
#define KEY_sub			206
#define KEY_substr		207
#define KEY_symlink		208
#define KEY_syscall		209
#define KEY_sysopen		210
#define KEY_sysread		211
#define KEY_system		212
#define KEY_syswrite		213
#define KEY_tell		214
#define KEY_telldir		215
#define KEY_tie			216
#define KEY_tied		217
#define KEY_time		218
#define KEY_times		219
#define KEY_tr			220
#define KEY_truncate		221
#define KEY_uc			222
#define KEY_ucfirst		223
#define KEY_umask		224
#define KEY_undef		225
#define KEY_unless		226
#define KEY_unlink		227
#define KEY_unpack		228
#define KEY_unshift		229
#define KEY_untie		230
#define KEY_until		231
#define KEY_use			232
#define KEY_utime		233
#define KEY_values		234
#define KEY_vec			235
#define KEY_wait		236
#define KEY_waitpid		237
#define KEY_wantarray		238
#define KEY_warn		239
#define KEY_while		240
#define KEY_write		241
#define KEY_x			242
#define KEY_xor			243
#define KEY_y			244
